'Mutant Monty.
'Amstrad CPC to modern hardware Conversion by Stephen Ancell.
'Project start date: 12th December 2013: 18:55 GMT. Completion date: TBC.
'Coded with Monkey-X, (www.monkey-x.com); an excellent cross-platform programming language from Blitz Research LTD.
'					(www.monkeycoder.co.nz)




'To Do: 
'Fix title music.
'Fix control latency
'Fix player sticking at right and bottom of screen.




Strict
Import mojo
Import diddy


'To find out if the game has been built in debug or release mode.
#if CONFIG = "debug"
	Const DEBUG_ENABLED:Bool = True
#else
	Const DEBUG_ENABLED:Bool = False
#endif


#if TARGET = "flash"
	Const fileExtAudio:String = ".mp3"
#endif


#if TARGET = "html5"
	Const fileExtAudio:String = ".wav"
#endif


'A few constants for choosing the gamestate.
Const LOADING:Int = 0
Const TITLE:Int = 1
Const GAME:Int = 2
Const PAUSE:Int = 3
Const DEMO:Int = 4
Const CONGRATS:Int = 5
Const GAME_OVER:Int = 6
Const TRUCK_CHASE:Int = 7
Const PLAYER_DIE:Int = 8
Const BONUS:Int = 9
Const FINAL_BONUS:Int = 10
Const CONGRATS_MUSIC:Int = 11
Const CONGRATS_MESSAGE:Int = 12


Global gameState:Int
Global showTutor:Int
Global congratsMessageState:Int
Global congratsTimer:TimerClass


'For debugging info.
Global messageStack:String[]


'Some arrays for storing clickable web addresses and any other clicky text.
Global addressList:LinkClass[]


'This bit is for storing keystates, so that the program can detect if a key has been released before the next press of that key.
Global keytappedRegister:Int[127]


'Size of the behind the scenes game area, before anything is scaled to suit the target screen size.
Global nativeWidth:Float
Global nativeHeight:Float


'Well... It would be pretty boring if all text had to be one size.
Global fontScaleX:Float = 1.00
Global fontScaleY:Float = 1.00


'All of the tiles for all of the game levels.
Global tileSheet:Image


'Stores a bunch of rectangles and their coordinates, for grabbing areas from the tilesheet for spamming onto the screen.
Global tileFrame:FrameClass[]


'The amount of columns and rows for tiles in a level.
Global tileMapColumnCount:Int
Global tileMapRowCount:Int


'Size of the tiles in a level.
Global tileWidth:Float
Global tileHeight:Float


'This is where the level data is stored
Global tileMap:Int[]


'A bit like the tilesheet but this time for sprites. :p
Global spriteSheet:Image
Global spriteFrame:FrameClass[]
Global spriteAnimTable:AnimationClass[]


'Stores zeros and ones, for Pixel Perfect Collision. :D
Global spriteCollisionData:Int[]


'Graphics overlay for the title screen.
Global titleGFX1:Image


'All the letters, numbers and symbols needed for the entire game.
Global imageFont:Image


'This is the storyline text that is scrolled on the title screen, and how far it has scrolled.
Global titleScroll:String
Global titleScrollProgress:Float


'How many demo screens that have been displayed, so that it knows when to switch back to the title screen; and the delay between each demo screen.
Global demoProgress:Int = 0
Global demoTimer:TimerClass


Global titleMusicState:Int = 1
Global musicOn:Int = 1


Global soundPickUp:Sound
Global soundPlayerDie:Sound
Global soundTruckChase:Sound
Global soundBeep:Sound
Global soundBonus:Sound


Global currentLevel:Int
Global lives:Int
Global levelTitle:String


Global player:PlayerClass
Global playerHide:Bool
Global directionKeyHoldStart:Float
Global playerDieing:Int
Global playerDieTimer:TimerClass
Global playerDieColorBurst:Int[] = [255, 255, 0, 0, 255, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255]


Global baddieList:BaddieClass[]
Global elevatorList:ElevatorClass[]
Global pickUpList:PickUpClass[]
Global exitDoor:ExitClass
Global damsel:DamselClass
Global truck:BaddieClass


Global score:Int
Global highScore:Int
Global bonusScore:Int
Global bonusTicker:TimerClass = New TimerClass(2000)


Global gameOverMessageGFX:Image
Global gameOverMessageProgress:Int
Global truckChaseProgress:Float
Global gameOverMessageTicker:TimerClass = New TimerClass(125)


Global pauseTimer:TimerClass = New TimerClass(500)


Const GAMESPEED:Float = 2.5
Global lastTime:Float = 0
Global timePassed:Float = 0
Global delta:Float = 0
Global deltaLow:Float = 0


Global gridOn:Int
Global gamePaused:Int


Global pauseGFX:Image
Global pauseScroll:Float


Global startButtonState:Int = 2
Global startButtonStateTimer:TimerClass = New TimerClass(500)




Class GameClass extends App
	Method OnCreate:Int()
		Seed = (Millisecs() + GetSeconds())
		gridOn = 0
		showTutor = 0
		score = 0
		highScore = 0
		bonusScore = 0
		gamePaused = 0
		pauseScroll = 0
		
		SetUpdateRate(60)
		
		LoadHighScore()
		SetGameState(TITLE)
		SetNativeValues()
		LoadGFX()
		LoadTitleScroll()
		LoadAudio()
		StartTitle()
		
		Return 0
	End
	
	
	Method OnUpdate:Int()
		if DEBUG_ENABLED = True
			'musicOn = 0
		EndIf
		
		
		timePassed = (Millisecs() - lastTime)
		lastTime = Millisecs()
		delta = (timePassed / 1000.0) * GAMESPEED
		
		if delta >= deltaLow
			deltaLow = delta
		EndIf
		
		if timePassed > 300
			timePassed = 0
			lastTime = Millisecs()
			delta = 0
		EndIf
		
		
		If GetGameState() = TITLE
			If startButtonStateTimer.GetTrigger() = True
				startButtonState = (startButtonState ~ 7)
			EndIf
			
			UpdateTitle()
		EndIf
		
		
		if GetGameState() = DEMO
			UpdateDemo()
		EndIf
		
		
		if GetGameState() = GAME
			if gamePaused = 0
				UpdateGame()
			EndIf
			
			if gamePaused = 1
				UpdatePause()
			EndIf
			
			if DEBUG_ENABLED = True
				if KeyTapped(KEY_1) = True
					gridOn = (gridOn ~ 1)
				EndIf
			EndIf
			
			if KeyTapped(KEY_P)
				gamePaused = (gamePaused ~ 1)
			EndIf
		EndIf
		
		
		if GetGameState() = BONUS
			UpdateBonus()
		EndIf
		
		
		if GetGameState() = FINAL_BONUS
			UpdateFinalBonus()
		EndIf
		
		
		if score > highScore
			highScore = score
		EndIf
		
		
		if GetGameState() = PLAYER_DIE
			UpdatePlayerDie()
		EndIf
		
		
		if GetGameState() = GAME_OVER
			UpdateGameOver()
		EndIf
		
		
		if GetGameState() = TRUCK_CHASE
			UpdateTruckChase()
		EndIf
		
		
		if GetGameState() = TITLE or GetGameState() = DEMO
			If KeyTapped(KEY_SPACE) = True or KeyTapped(KEY_LMB) = True
				DeleteAllLinks()
				NewGame()
			EndIf
			
			if KeyTapped(KEY_R) = True
				ResetHighScore()
			EndIf
		EndIf
		
		
		if KeyTapped(KEY_M) = True
			musicOn = (musicOn ~ 1)
		EndIf
		
		
		if GetGameState() = CONGRATS or GetGameState() = CONGRATS_MUSIC or GetGameState() = CONGRATS_MESSAGE
			UpdateCongrats()
		EndIf
		
		
		if GetGameState() <> GAME_OVER And GetGameState() <> TRUCK_CHASE And GetGameState() <> BONUS
			PlayTitleMusic()
		EndIf
		
		
		Return 0
	End
	
	
	Method OnRender:Int()
		if GetGameState() = TITLE
			DrawTitle()
			
			
			if DEBUG_ENABLED = True
				DrawText("Debug enabled!", 0, 0)
				DrawText("Press M to mute music", 0, 16)
				DrawText("Z / X to skip through levels", 0, 32)
				DrawText("All text here and above is temporary during debug mode", 0, 48)
			EndIf
		EndIf
		
		
		if GetGameState() = DEMO
			DrawDemo()
		EndIf
		
		
		if GetGameState() = GAME or GetGameState() = CONGRATS_MUSIC or GetGameState() = PAUSE
			DrawGame()
			
			if gamePaused = 1
				DrawPause()
			EndIf
			
			if currentLevel > 40 And DEBUG_ENABLED = True
				SetColor(255, 255, 255)
				DrawText("Target Direction: " + player.GetTargetDirection(), 300, 0)
				DrawText("Current Direction: " + player.GetDirection(), 300, 15)
				DrawText("Player X: " + player.GetX(), 300, 30)
				DrawText("Player Y" + player.GetY(), 300, 45)
			EndIf
		EndIf
		
		
		if GetGameState() = BONUS or GetGameState() = FINAL_BONUS
			DrawBonus()
		EndIf
		
		
		if GetGameState() = PLAYER_DIE
			DrawPlayerDie()
		EndIf
		
		
		if GetGameState() = CONGRATS or GetGameState() = CONGRATS_MESSAGE
			DrawCongrats()
		EndIf
		
		
		if GetGameState() = GAME_OVER
			DrawGameOver()
		EndIf
		
		
		if GetGameState() = TRUCK_CHASE
			DrawTruchChase()
		EndIf
		
		
		ShowVars()
		ShowMessages(0, 360)
		ClearMessages()
		
		Return 0
	End
End




Class LinkClass
	Field x:Float
	Field y:Float
	Field address:string
	Field directives:String
	Field scaleX:Float
	Field scaley:Float
	
	
	Method New(address:String, scaleX:Float = 1.00, scaleY:Float = 1.00, directives:String = "")
		Self.x = x
		Self.y = y
		Self.address = address
		Self.directives = directives
		Self.scaleX = scaleX
		Self.scaley = scaleY
	End
	
	
	Method SetPosition:Void(x:Float, y:Float)
		Self.x = x
		Self.y = y
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method GetAddress:String()
		Return Self.address
	End
	
	
	Method GetDirectives:String()
		Return Self.directives
	End
	
	
	Method Draw:Void(directives:String = "")
		Local highlight:String = ""
		
		
		SetFontScale(Self.scaleX, Self.scaley)
		
		if HoverString(GetAddress(), GetX(), GetY(), GetDirectives()) = True
			highlight = "highlight"
		EndIf
		
		DrawString(4, GetAddress(), GetX(), GetY(), GetDirectives() + highlight)
	End
	
	
	Method Clicked:Bool()
		Local flag:Bool = False
		
		
		if HoverString(GetAddress(), GetX(), GetY(), GetDirectives()) = True
			if MouseHit(MOUSE_LEFT) = True
				flag = True
			EndIf
		EndIf
		
		Return flag
	End
End




Class PlayerClass
	Field x:Float
	Field y:Float
	Field speed:Float
	Field speedLow:Float
	Field speedHigh:Float
	Field animSpeedLow:Float
	Field animSpeedHigh:Float
	Field acceleration:Float
	Field animTable:Int[]
	Field direction:Int
	Field targetDirection:Int
	Field canMove:Bool
	Field searchKey:String[] = ["", "MontyUp", "MontyRight", "MontyDown", "MontyLeft"]
	
	
	Method New()
		SetAnimationKeys()
		Self.x = 0
		Self.y = 0
		Self.acceleration = 0
		Self.canMove = True
	End
	
	
	Method SetAnimationKeys:Void()
		Local index:Int
		Local searchIndex:Int
		
		
		Self.animTable = Self.animTable.Resize(5)
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			For searchIndex = 1 to 4
				if spriteAnimTable[index].GetName() = Self.searchKey[searchIndex]
					Self.animTable[searchIndex] = index
				EndIf
			Next
		Next
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
	End
	
	
	Method SetAnimSpeed:Void(timeDelay:Float)
		Local index:Int
		
		
		For index = 0 to (Self.animTable.Length() - 1)
			spriteAnimTable[animTable[index]].SetFrameDelay(timeDelay)
		Next
	End
	
	
	Method SetDirectionByName:Void(dirName:String)
		Local searchIndex:Int
		
		
		For searchIndex = 1 to 4
			if searchKey[searchIndex] = dirName
				Self.direction = searchIndex
			EndIf
		Next
	End
	
	
	Method SetDirectionByIndex:Void(dirIndex:Int)
		self.direction = dirIndex
	End
	
	
	Method SetTargetDirectionByName:Void(dirName:String)
		Local searchIndex:Int
		
		
		For searchIndex = 1 to 4
			if searchKey[searchIndex] = dirName
				Self.targetDirection = (searchIndex)
			EndIf
		Next
	End
	
	
	Method SetTargetDirectionByIndex:Void(dirIndex:Int)
		Self.targetDirection = dirIndex
	End
	
	
	Method SetPosition:Void(x:Float, y:Float)
		SetX(x)
		SetY(y)
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method InlineWithTileMapColumn:Bool()
		if GetX() Mod tileWidth = 0.00
			Return True
		Else
			Return False
		EndIf
	End
	
	
	Method InlineWithTileMapRow:Bool()
		if GetY() Mod tileHeight = 0.00
			Return True
		Else
			Return False
		EndIf
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
	End
	
	
	Method GetTargetDirection:Int()
		Return Self.targetDirection
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method Update:Void()
		Local pixelsPerSecond:Float = (100 * delta)
		Local gridRefX:Int = (Floor(GetX()) / tileWidth)
		Local gridRefY:Int = (Floor(GetY()) / tileHeight)
		
		
		if GetDirection() = 1
			SetY(GetY() - GetSpeed() * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 2
			SetX(GetX() + GetSpeed() * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 3
			SetY(GetY() + GetSpeed() * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 4
			SetX(GetX() - GetSpeed() * pixelsPerSecond)
		EndIf
	End
	
	
	Method Draw:Void()
		DrawSpriteAnim(animTable[GetDirection()], GetX(), GetY(), False, False)
	End
End




Class BaddieClass
	Field x:Float
	Field y:Float
	Field anim:Int
	Field direction:Int
	Field targetDirection1:Int
	Field targetDirection2:Int
	Field targetX1:Float
	Field targetY1:Float
	Field targetX2:Float
	Field targetY2:Float
	Field speed:Float
	
	
	Method New()
		Self.x = 0
		Self.y = 0
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method SetTargetX1:Void(x:Float)
		Self.targetX1 = x
	End
	
	
	Method SetTargetY1:Void(y:Float)
		Self.targetY1 = y
	End
	
	
	Method SetTargetX2:Void(x:Float)
		Self.targetX2 = x
	End
	
	
	Method SetTargetY2:Void(y:Float)
		Self.targetY2 = y
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method GetTargetX1:Float()
		Return Self.targetX1
	End
	
	
	Method GetTargetY1:Float()
		Return Self.targetY1
	End
	
	
	Method GetTargetX2:Float()
		Return Self.targetX2
	End
	
	
	Method GetTargetY2:Float()
		Return Self.targetY2
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
	End
	
	
	Method SetAnimation:Void(animName:String)
		Local index:Int
		
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			if spriteAnimTable[index].GetName() = animName
				Self.anim = index
			EndIf
		Next
	End
	
	
	Method SetDirectionByIndex:Void(dirIndex:Int)
		Self.direction = dirIndex
	End
	
	
	Method SetTargetDirection1:Void(dirIndex:Int)
		Self.targetDirection1 = dirIndex
	End
	
	
	Method SetTargetDirection2:Void(dirIndex:Int)
		Self.targetDirection2 = dirIndex
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
	End
	
	
	Method GetTargetDirection1:Int()
		Return Self.targetDirection1
	End
	
	
	Method GetTargetDirection2:Int()
		Return Self.targetDirection2
	End
	
	
	Method Update:Void()
		Local pixelsPerSecond:Float = (100 * delta)
		
		
		if GetDirection() = 1
			SetY(GetY() - speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 2
			SetX(GetX() + speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 3
			SetY(GetY() + speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 4
			SetX(GetX() - speed * pixelsPerSecond)
		EndIf
		
		Restrict()
	End
	
	
	Method Restrict:Void()
		if GetX() <= GetTargetX1() And GetY() <= GetTargetY1()
			SetDirectionByIndex(GetTargetDirection1())
			SetX(GetTargetX1())
			SetY(GetTargetY1())
		EndIf
		
		if GetX() >= GetTargetX2() And GetY() >= GetTargetY2()
			SetDirectionByIndex(GetTargetDirection2())
			SetX(GetTargetX2())
			SetY(GetTargetY2())
		EndIf
	End
	
	
	Method Draw:Void()
		Local reverseX:Bool = False
		Local reverseY:Bool = False
		
'		if GetDirection() = 4
'			reverseX = True
'		EndIf
		
		DrawSpriteAnim(Self.anim, GetX(), GetY(), reverseX, reverseY)
	End
End




Class ElevatorClass
	Field x:Float
	Field y:Float
	Field anim:Int
	Field direction:Int
	Field targetDirection1:Int
	Field targetDirection2:Int
	Field targetX1:Float
	Field targetY1:Float
	Field targetX2:Float
	Field targetY2:Float
	Field speed:Float
	
	
	Method New()
		Self.x = 0
		Self.y = 0
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method SetTargetX1:Void(x:Float)
		Self.targetX1 = x
	End
	
	
	Method SetTargetY1:Void(y:Float)
		Self.targetY1 = y
	End
	
	
	Method SetTargetX2:Void(x:Float)
		Self.targetX2 = x
	End
	
	
	Method SetTargetY2:Void(y:Float)
		Self.targetY2 = y
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method GetTargetX1:Float()
		Return Self.targetX1
	End
	
	
	Method GetTargetY1:Float()
		Return Self.targetY1
	End
	
	
	Method GetTargetX2:Float()
		Return Self.targetX2
	End
	
	
	Method GetTargetY2:Float()
		Return Self.targetY2
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
	End
	
	
	Method SetAnimation:Void(animName:String)
		Local index:Int
		
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			if spriteAnimTable[index].GetName() = animName
				Self.anim = index
			EndIf
		Next
	End
	
	
	Method SetDirectionByIndex:Void(dirIndex:Int)
		Self.direction = dirIndex
	End
	
	
	Method SetTargetDirection1:Void(dirIndex:Int)
		Self.targetDirection1 = dirIndex
	End
	
	
	Method SetTargetDirection2:Void(dirIndex:Int)
		Self.targetDirection2 = dirIndex
	End
	
	
	Method GetDirection:Int()
		Return Self.direction
	End
	
	
	Method GetTargetDirection1:Int()
		Return Self.targetDirection1
	End
	
	
	Method GetTargetDirection2:Int()
		Return Self.targetDirection2
	End
	
	
	Method Update:Void()
		Local pixelsPerSecond:Float = (100 * delta)
		
		
		if GetDirection() = 1
			SetY(GetY() - speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 2
			SetX(GetX() + speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 3
			SetY(GetY() + speed * pixelsPerSecond)
		EndIf
		
		if GetDirection() = 4
			SetX(GetX() - speed * pixelsPerSecond)
		EndIf
		
		Restrict()
	End
	
	
	Method Restrict:Void()
		if GetX() <= GetTargetX1() And GetY() <= GetTargetY1()
			SetDirectionByIndex(GetTargetDirection1())
			SetX(GetTargetX1())
			SetY(GetTargetY1())
		EndIf
		
		if GetX() >= GetTargetX2() And GetY() >= GetTargetY2()
			SetDirectionByIndex(GetTargetDirection2())
			SetX(GetTargetX2())
			SetY(GetTargetY2())
		EndIf
	End
	
	
	Method Draw:Void()
		DrawCable()
		DrawSpriteAnim(Self.anim, GetX(), GetY(), False, False)
	End
	
	Method DrawCable:Void()
		SetColor(255, 255, 255)
		DrawLine((GetTargetX1() + (spriteFrame[Self.anim].GetWidth() / 2)) * GetScaleX(), (GetTargetY1() - (spriteFrame[Self.anim].GetHeight() / 3)) * GetScaleY(), (GetX() + (spriteFrame[Self.anim].GetWidth() / 2)) * GetScaleX(), GetY() * GetScaleY())
	End
End




Class PickUpClass
	Field x:Float
	Field y:Float
	Field anim:Int
	
	
	Method New(x:Float, y:Float, animName:String)
		Self.x = x
		Self.y = y
		SetAnimation(animName)
	End
	
	
	Method SetAnimation:Void(animName:String)
		Local index:Int
		
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			if spriteAnimTable[index].GetName() = animName
				Self.anim = index
			EndIf
		Next
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method Draw:Void()
		DrawSpriteAnim(Self.anim, GetX(), GetY(), False, False)
	End
End




Class ExitClass
	Field x:Float
	Field y:Float
	Field anim:Int
	Field animName:String
	
	
	Method New()
		
	End
	
	
	Method SetAnimation:Void(animName:String)
		Local index:Int
		
		
		Self.animName = animName
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			if spriteAnimTable[index].GetName() = animName
				Self.anim = index
			EndIf
		Next
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method IsOpen:Bool()
		Local flag:Bool = False
		
		
		if Self.animName = "ExitOpen"
			flag = True
		EndIf
		
		Return flag
	End
	
	
	Method Draw:Void()
		DrawSpriteAnim(Self.anim, GetX(), GetY(), False, False)
	End
End




Class DamselClass
	Field x:Float
	Field y:Float
	Field anim:Int
	
	
	Method New()
		
	End
	
	
	Method SetAnimation:Void(animName:String)
		Local index:Int
		
		
		For index = 0 to (spriteAnimTable.Length() - 1)
			if spriteAnimTable[index].GetName() = animName
				Self.anim = index
			EndIf
		Next
	End
	
	
	Method SetX:Void(x:Float)
		Self.x = x
	End
	
	
	Method SetY:Void(y:Float)
		Self.y = y
	End
	
	
	Method GetX:Float()
		Return Self.x
	End
	
	
	Method GetY:Float()
		Return Self.y
	End
	
	
	Method Draw:Void()
		DrawSpriteAnim(Self.anim, GetX(), GetY(), False, False)
	End
End




Class FrameClass
	Field fX:Int
	Field fY:Int
	Field fWidth:Int
	Field fHeight:Int
	
	
	Method New(x:Int, y:Int, fWidth:Int, fHeight:Int)
		Self.fX = x
		Self.fY = y
		Self.fWidth = fWidth
		Self.fHeight = fHeight
	End
	
	
	Method GetX:Int()
		Return Self.fX
	End
	
	
	Method GetY:Int()
		Return Self.fY
	End
	
	
	Method GetWidth:Int()
		Return Self.fWidth
	End
	
	
	Method GetHeight:Int()
		Return Self.fHeight
	End
End




Class AnimationClass
	Field name:String
	Field speed:Float
	Field speedLow:Float
	Field speedHigh:Float
	Field sequence:Int[]
	Field currentFrame:Int
	Field frameTimer:TimerClass
	Field frameDelayLow:Float
	Field frameDelayHigh:Float
	
	
	Method New(name:String, frameDelay:Float)
		SetName(name)
		Self.currentFrame = 0
		Self.frameTimer = New TimerClass(frameDelay)
	End
	
	
	Method Update:Void()
		if Self.frameTimer.GetTrigger() = True
			Self.currentFrame = (Self.currentFrame + 1)
		EndIf
		
		if Self.currentFrame > Self.sequence.Length() - 1
			Self.currentFrame = 0
		EndIf
	End
	
	
	Method SetName:Void(name:String)
		Self.name = name
	End
	
	
	Method SetSpeed:Void(speed:Float)
		Self.speed = speed
	End
	
	
	Method AddFrame:Void(index:Int)
		Self.sequence = Self.sequence.Resize(Self.sequence.Length() + 1)
		Self.sequence[Self.sequence.Length() - 1] = index
	End
	
	
	Method SetFrameDelayLow:Void()
		SetFrameDelay(Self.frameDelayLow)
	End
	
	
	Method SetFrameDelayHigh:Void()
		SetFrameDelay(Self.frameDelayHigh)
	End
	
	
	Method SetFrameDelay:Void(timeDelay:Float)
		Self.frameTimer.SetDelay(timeDelay)
	End
	
	
	Method GetName:String()
		Return Self.name
	End
	
	
	Method GetSpeed:Float()
		Return Self.speed
	End
	
	
	Method GetCurrentFrame:Int()
		Return Self.currentFrame
	End
	
	
	Method GetFrame:Int(reference:Int)
		Return Self.sequence[reference]
	End
	
	
	Method GetCurrentFrameContent:Int()
		Return Self.sequence[Self.GetCurrentFrame()]
	End
	
	
	Method GetFrameCount:Int()
		Return Self.sequence.Length()
	End
	
	
	Method GetFinalFrameReference:Int()
		Return Self.sequence.Length() - 1
	End
End




Class TimerClass
	Field delay:Float
	Field lastTime:Float
	
	
	Method New(delay:Float)
		SetDelay(delay)
		Self.lastTime = Millisecs()
	End
	
	
	Method GetTrigger:Bool()
		Local trigger:Bool
		
		
		if Millisecs() >= (Self.lastTime + Self.delay)
			Self.lastTime = Millisecs()
			trigger = True
		Else
			trigger = False
		EndIf
		
		Return trigger
	End
	
	
	Method SetDelay:Void(delay:Float)
		Self.delay = delay
		'Self.lastTime = Millisecs()
	End
	
	
	Method GetDelay:Float()
		Return Self.delay
	End
End




Class FileLoaderClass
	Field fileData:String[]
	Field currentLine:Int
	
	
	Method New(fileName:String)
		fileData = LoadString(fileName).Split("~n")
		currentLine = 0
	End
	
	
	Method ReadLine:String()
		Local currentData:String
		
		
		if Eof() = False
			currentData = Self.fileData[Self.currentLine]
			currentLine = (currentLine + 1)
		Else
			currentData = "Data Exhausted!"
		EndIf
		
		Return currentData.Trim()
	End
	
	
	Method Eof:Bool()
		if Self.currentLine > (fileData.Length() - 1)
			Return True
			
		Else
			Return False
		EndIf
	End
	
	
	Method Empty:Void()
		Local counter:Int
		
		
		For counter = 0 to (fileData.Length() - 1)
			fileData[counter] = ""
		Next
		
		fileData = fileData.Resize(0)
	End
End




Function Main:Int()
	New GameClass()
	Return 0
End




Function SaveHighScore:Void()
	SaveState(highScore)
End




Function LoadHighScore:Void()
	Local state:String = LoadState()
	
	
	if state
		bonusScore = Int(state.Trim())
	Else
		bonusScore = 0
	EndIf
End




Function ResetHighScore:Void()
	highScore = 0
	SaveHighScore()
End




Function LoadGFX:Void()
	LoadTitleGFX()
	LoadTileSheet()
	LoadSpriteSheet()
	LoadSpriteAnimTable()
	LoadSpriteCollisionData()
	LoadImageFonts()
	
	gameOverMessageGFX = LoadImage("graphics/gameover.png")
	pauseGFX = LoadImage("graphics/pause.png")
End




Function LoadImageFonts:Void()
	imageFont = LoadImage("graphics/imagefont.png")
End




Function LoadAudio:Void()
	soundPickUp = LoadSound("audio/pickup" + fileExtAudio)
	soundPlayerDie = LoadSound("audio/playerdie" + fileExtAudio)
	soundTruckChase = LoadSound("audio/truckchase" + fileExtAudio)
	soundBeep = LoadSound("audio/beep" + fileExtAudio)
	soundBonus = LoadSound("audio/bonus" + fileExtAudio)
End




Function LoadTileSheet:Void()
	Local column:Int
	Local row:Int
	Local columnCount:Int
	Local rowCount:Int
	Local index:Int = 0
	
	
	tileSheet = LoadImage("graphics/tiles.png")
	
	columnCount = (tileSheet.Width() / tileWidth)
	rowCount = (tileSheet.Height() / tileHeight)
	
	tileFrame = tileFrame.Resize(columnCount * rowCount)
	
	For row = 0 to (rowCount - 1)
		For column = 0 to (columnCount - 1)
			index = column + (columnCount * row)
			tileFrame[index] = New FrameClass(tileWidth * column, tileHeight * row, tileWidth, tileHeight)
		Next
	Next
End




Function LoadSpriteSheet:Void()
	Local rFile:FileLoaderClass
	Local fileData:String
	
	
	spriteSheet = LoadImage("graphics/spritesheet.png")
	rFile = ReadFile("graphics/spritemap.dat")
	spriteFrame = spriteFrame.Resize(spriteFrame.Length() + 1)
	
	Repeat
		fileData = rFile.ReadLine()
		
		if fileData.Contains("//") = False
			if fileData = "FrameStart"
				spriteFrame = spriteFrame.Resize(spriteFrame.Length() + 1)
				spriteFrame[spriteFrame.Length() - 1] = New FrameClass(INT(rFile.ReadLine()), INT(rFile.ReadLine()), INT(rFile.ReadLine()), INT(rFile.ReadLine()))
			EndIf
		EndIf
	Until rFile.Eof() = True
	
	CloseFile(rFile)
	fileData = ""
End




Function LoadSpriteCollisionData:Void()
	Local rFile:FileLoaderClass
	Local fileData:String
	Local dataWidth:Int
	Local dataHeight:Int
	Local x:Int
	Local y:Int
	Local index:Int
	
	
	rFile = ReadFile("graphics/spritecollisionmask.dat")
	dataWidth = Int(rFile.ReadLine())
	dataHeight = Int(rFile.ReadLine())
	spriteCollisionData = spriteCollisionData.Resize(dataWidth * dataHeight)
	
	For y = 0 to (dataHeight - 1)
		fileData = rFile.ReadLine().Trim()
		
		For x = 0 to (dataWidth - 1)
			index = x + (dataWidth * y)
			spriteCollisionData[index] = Int(Mid(fileData, x + 1, 1))
		Next
	Next
	
	CloseFile(rFile)
	fileData = ""
End




Function LoadSpriteAnimTable:Void()
	Local rFile:FileLoaderClass
	Local fileData:String
	
	
	rFile = ReadFile("graphics/spriteanimtable.dat")
	spriteFrame = spriteFrame.Resize(spriteFrame.Length() + 1)
	
	Repeat
		fileData = rFile.ReadLine()
		
		if fileData.Contains("//") = False
			if fileData = "AnimStart"
				spriteAnimTable = spriteAnimTable.Resize(spriteAnimTable.Length() + 1)
				spriteAnimTable[spriteAnimTable.Length() - 1] = New AnimationClass(rFile.ReadLine(), Float(rFile.ReadLine()))
				
				Repeat
					fileData = rFile.ReadLine()
					
					if fileData = "AnimEnd"
						Exit
					EndIf
					
					spriteAnimTable[spriteAnimTable.Length() - 1].AddFrame(Int(fileData))
				Forever
			EndIf
		EndIf
	Until rFile.Eof() = True
	
	CloseFile(rFile)
	fileData = ""
End




Function LoadTitleGFX:Void()
	titleGFX1 = LoadImage("graphics/titlescreen.png")
End




Function LoadTitleScroll:Void()
	titleScrollProgress = 590.00
	titleScroll = ""
	titleScroll += "GUIDE MONTY THROUGH THE FORTY EXQUISITE CAVES, MAZES AND BUILDINGS."
	titleScroll += " "
	titleScroll += "COLLECT ALL THE GOLD BUT BEWARE THE EVIL ALIENS WHO HOLD THE FAIR MAIDEN CAPTIVE..."
	titleScroll += " "
	titleScroll += "GOOD LUCK..."
End




Function ResetTitleScroll:Void()
	titleScrollProgress = 0.00
End




Function LoadLevel:Void(levelNumber:Int)
	Local fileData:String
	Local rFile:FileLoaderClass
	
	
	tileMapColumnCount = 16
	tileMapRowCount = 10
	rFile = ReadFile("levels/level" + levelNumber + ".dat")
	
	Repeat
		fileData = rFile.ReadLine()
		
		if fileData.Contains("LevelTitle")
			SetLevelTitle(fileData)
		EndIf
		
		if fileData.Contains("LevelStart")
			LoadTileMap(rFile)
		EndIf
		
		if fileData.Contains("PlayerStart")
			LoadPlayer(rFile)
		EndIf
		
		if fileData.Contains("BaddieStart")
			LoadBaddie(rFile)
		EndIf
		
		if fileData.Contains("ExitStart")
			LoadExit(rFile)
		EndIf
		
		if fileData.Contains("DamselStart")
			LoadDamsel(rFile)
		EndIf
		
		if fileData.Contains("ElevatorStart")
			LoadElevator(rFile)
		EndIf
		
		if fileData.Contains("HidePlayer")
			playerHide = True
		EndIf
	Until rFile.Eof() = True
	
	CloseFile(rFile)
	fileData = ""
End


Function SetLevelTitle:Void(fileData:String)
	levelTitle = fileData[fileData.Find(" ") ..].Trim()
End


Function GetLevelTitle:String()
	Return levelTitle
End


Function LoadTileMap:Void(rFile:FileLoaderClass)
	Local columnCount:Int = tileMapColumnCount
	Local rowCount:Int = tileMapRowCount
	Local column:Int
	Local row:Int
	Local index:Int
	Local fileData:String
	
	
	tileMap = tileMap.Resize(columnCount * rowCount)
	
	For row = 0 to (rowCount - 1)
		fileData = rFile.ReadLine().Trim()
		
		For column = 0 to (columnCount - 1)
			index = column + (columnCount * row)
			tileMap[index] = HexToInt(Mid(fileData, (column * 2) + 1, 2))
		Next
	Next
	
	fileData = ""
End




Function LoadPlayer:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local animName:String
	Local x:Float
	Local y:Float
	Local direction:Int
	Local speed:Float
	Local speedHigh:Float
	Local speedLow:Float
	Local animFast:Float
	Local animSlow:Float
	
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	direction = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	speed = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	speedHigh = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	speedLow = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	animFast = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	animSlow = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnPlayer(x, y, animName, speed, speedHigh, speedLow, animFast, animSlow)
End




Function LoadBaddie:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local animName:String
	Local x:Float
	Local y:Float
	Local targetX1:Float
	Local targetY1:Float
	Local targetX2:Float
	Local targetY2:Float
	Local direction:Int
	Local targetDirection1:Int
	Local targetDirection2:Float
	Local speed:Float
	
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	direction = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetX1 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetY1 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetDirection1 = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetX2 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetY2 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetDirection2 = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	speed = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnBaddie(x, y, animName, targetX1, targetY1, targetX2, targetY2, direction, targetDirection1, targetDirection2, speed)
End




Function LoadElevator:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local animName:String
	Local x:Float
	Local y:Float
	Local targetX1:Float
	Local targetY1:Float
	Local targetX2:Float
	Local targetY2:Float
	Local direction:Int
	Local targetDirection1:Int
	Local targetDirection2:Float
	Local speed:Float
	
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	direction = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetX1 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetY1 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetDirection1 = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetX2 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetY2 = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	targetDirection2 = Int(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	speed = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnElevator(x, y, animName, targetX1, targetY1, targetX2, targetY2, direction, targetDirection1, targetDirection2, speed)
End




Function LoadPickUps:Void(levelNumber:Int)
	Local fileData:String
	Local rFile:FileLoaderClass
	
	
	tileMapColumnCount = 16
	tileMapRowCount = 10
	rFile = ReadFile("levels/level" + levelNumber + ".dat")
	
	Repeat
		fileData = rFile.ReadLine()
		
		IF fileData.Contains("PickUpStart")
			LoadPickUp(rFile)
		EndIf
	Until rFile.Eof() = True
	
	CloseFile(rFile)
	fileData = ""
End




Function LoadPickUp:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local x:Float
	Local y:Float
	Local animName:String
	
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnPickUp(x, y, animName)
End




Function LoadExit:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local animName:String
	Local x:Float
	Local y:Float
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnExit(x, y, animName)
End




Function LoadDamsel:Void(rFile:FileLoaderClass)
	Local fileData:String
	Local animName:String
	Local x:Float
	Local y:Float
	
	fileData = rFile.ReadLine()
	animName = fileData[fileData.Find(" ") ..].Trim()
	
	fileData = rFile.ReadLine()
	x = Float(fileData[fileData.Find(" ") ..].Trim())
	
	fileData = rFile.ReadLine()
	y = Float(fileData[fileData.Find(" ") ..].Trim())
	
	SpawnDamsel(x, y, animName)
End




Function DrawTileMap:Void(x:Float, y:Float)
	Local columnCount:Int = tileMapColumnCount
	Local rowCount:Int = tileMapRowCount
	Local column:Int
	Local row:Int
	Local index:Int
	
	
	For row = 0 to (rowCount - 1)
		For column = 0 to (columnCount - 1)
'			index = column + (columnCount * row)
'			DrawTileFrame(tileMap[index], column * tileWidth, row * tileHeight)
			DrawTileFrame(GetTileInfo(column, row), column * tileWidth, row * tileHeight)
		Next
	Next
End




Function DrawGrid:Void(x:Float, y:Float)
	Local columnCount:Int = tileMapColumnCount
	Local rowCount:Int = tileMapRowCount
	Local column:Int
	Local row:Int
	Local index:Int
	
	
	For row = 0 to (rowCount - 1)
		For column = 0 to (columnCount - 1)
			DrawRectOutline(column * tileWidth, row * tileHeight, tileWidth, tileHeight)
		Next
	Next
End




Function DrawBaddies:Void()
	Local index:Int
	
	
	For index = 0 to (baddieList.Length() - 1)
		baddieList[index].Draw()
	Next
End




Function DrawElevators:Void()
	Local index:Int
	
	
	For index = 0 to (elevatorList.Length() - 1)
		elevatorList[index].Draw()
	Next
End




Function DrawPickUps:Void()
	Local index:Int
	
	
	For index = 0 to (pickUpList.Length() - 1)
		if pickUpList[index] <> Null
			pickUpList[index].Draw()
		EndIf
	Next
End




Function UpdateBaddies:Void()
	Local index:Int
	
	
	For index = 0 to (baddieList.Length() - 1)
		baddieList[index].Update()
	Next
End




Function UpdateElevators:Void()
	Local index:Int
	
	
	For index = 0 to (elevatorList.Length() - 1)
		elevatorList[index].Update()
	Next
End




Function GetTileInfo:Int(column:Int, row:Int)
	Return tileMap[column + (tileMapColumnCount * row)]
End




Function DrawSpriteAnim:Void(index:Int, x:Float, y:Float, reverseX:Bool, reverseY:Bool)
	Local spriteAnim:AnimationClass
	Local currentFrame:Int
	
	
	spriteAnim = spriteAnimTable[index]
	currentFrame = spriteAnim.sequence[spriteAnim.GetCurrentFrame()]
	DrawSpriteFrame(currentFrame, x, y, reverseX, reverseY)
End




Function DrawFrameCollision:Void(index:Int, x:Float, y:Float)
	Local frame1:FrameClass
	Local iX:Int
	Local iY:Int
	
	
	frame1 = spriteFrame[index]
	
	For iY = 0 to (frame1.GetHeight() - 1)
		For iX = 0 to (frame1.GetWidth() - 1)
			if GetSpriteCollisionPixel(index, iX, iY) = 1
				DrawPoint(iX + x, iY + y)
			EndIf
		Next
	Next
End




Function SpriteFramesCollide:Bool(index1:Int, x1:Float, y1:Float, index2:Int, x2:Float, y2:Float)
	Local frame1:FrameClass
	Local frame2:FrameClass
	Local iX:Int
	Local iY:Int
	Local pixel1:Int
	Local pixel2:Int
	Local flag:Bool = False
	
	
	frame1 = spriteFrame[index1]
	frame2 = spriteFrame[index2]
	
	if x1 > (x2 - frame1.GetWidth()) And x1 < (x2 + frame2.GetWidth())
		if y1 > (y2 - frame1.GetHeight()) And y1 < (y2 + frame2.GetHeight())
			For iY = Floor(y1) to ((Floor(y1) + frame1.GetHeight()) - 1)
				For iX = Floor(x1) to ((Floor(x1) + frame1.GetWidth()) - 1)
					if iX > x2 And iX < (x2 + frame2.GetWidth())
						if iY > y2 And iY < (y2 + frame2.GetHeight())
							pixel1 = GetSpriteCollisionPixel(index1, iX - x1, iY - y1)
							pixel2 = GetSpriteCollisionPixel(index2, iX - x2, iY - y2)
							
							if pixel1 = 1 And pixel2 = 1
								flag = True
							EndIf
						EndIf
					EndIf
				Next
			Next
		EndIf
	EndIf
	
	Return flag
End




Function GetSpriteCollisionPixel:Int(frameIndex:Int, x:Int, y:Int)
	Local iX:Int
	Local iY:Int
	
	
	iX = (spriteFrame[frameIndex].GetX() + x)
	iY = (spriteFrame[frameIndex].GetY() + y)
	
	Return spriteCollisionData[iX + (spriteSheet.Width() * iY)]
End




Function DrawSpriteFrame:Void(index:Int, x:Float, y:Float, reverseX:Bool, reverseY:Bool)
	Local scaleX:Float = GetScaleX()
	Local scaleY:Float = GetScaleY()
	
	
	if reverseX = True
		scaleX = -scaleX
	EndIf
	
	DrawImageRect(spriteSheet, x * GetScaleX(), y * GetScaleY(), spriteFrame[index].GetX(), spriteFrame[index].GetY(), spriteFrame[index].GetWidth(), spriteFrame[index].GetHeight(), 0, scaleX, scaleY)
End




Function DrawTileFrame:Void(index:Int, x:Float, y:Float)
	if index > 0
		DrawImageRect(tileSheet, x * GetScaleX(), y * GetScaleY(), tileFrame[index].GetX(), tileFrame[index].GetY(), tileFrame[index].GetWidth(), tileFrame[index].GetHeight(), 0, GetScaleX(), GetScaleY())
	EndIf
End




Function UpdateSpriteAnims:Void()
	Local anim:AnimationClass
	
	
	For anim = EachIn spriteAnimTable
		anim.Update()
	Next
End




Function SpawnPlayer:Void(x:Float, y:Float, directionByName:String, speed:Float, speedHigh:Float, speedLow:Float, animFast:Float, animSlow:Float)
	if player = Null
		player = New PlayerClass()
	EndIf
	
	player.SetPosition(x, y)
	player.SetTargetDirectionByName(directionByName)
	player.SetDirectionByIndex(player.GetTargetDirection())
	player.SetSpeed(speed)
	player.speedHigh = speedHigh
	player.speedLow = speedLow
	player.animSpeedHigh = animFast
	player.animSpeedLow = animSlow
End




Function SpawnBaddie:Void(x:Float, y:Float, animName:String, targetX1:Float, targetY1:Float, targetX2:Float, targetY2:Float, direction:Int, targetDirection1:Int, targetDirection2:Int, speed:Float)
	baddieList = baddieList.Resize(baddieList.Length() + 1)
	baddieList[baddieList.Length() - 1] = New BaddieClass()
	
	baddieList[baddieList.Length() - 1].SetAnimation(animName)
	baddieList[baddieList.Length() - 1].SetX(x)
	baddieList[baddieList.Length() - 1].SetY(y)
	baddieList[baddieList.Length() - 1].SetTargetX1(targetX1)
	baddieList[baddieList.Length() - 1].SetTargetY1(targetY1)
	baddieList[baddieList.Length() - 1].SetTargetX2(targetX2)
	baddieList[baddieList.Length() - 1].SetTargetY2(targetY2)
	baddieList[baddieList.Length() - 1].SetDirectionByIndex(direction)
	baddieList[baddieList.Length() - 1].SetTargetDirection1(targetDirection1)
	baddieList[baddieList.Length() - 1].SetTargetDirection2(targetDirection2)
	baddieList[baddieList.Length() - 1].SetSpeed(speed)
End




Function SpawnElevator:Void(x:Float, y:Float, animName:String, targetX1:Float, targetY1:Float, targetX2:Float, targetY2:Float, direction:Int, targetDirection1:Int, targetDirection2:Int, speed:Float)
	elevatorList = elevatorList.Resize(elevatorList.Length() + 1)
	elevatorList[elevatorList.Length() - 1] = New ElevatorClass()
	
	elevatorList[elevatorList.Length() - 1].SetAnimation(animName)
	elevatorList[elevatorList.Length() - 1].SetX(x)
	elevatorList[elevatorList.Length() - 1].SetY(y)
	elevatorList[elevatorList.Length() - 1].SetTargetX1(targetX1)
	elevatorList[elevatorList.Length() - 1].SetTargetY1(targetY1)
	elevatorList[elevatorList.Length() - 1].SetTargetX2(targetX2)
	elevatorList[elevatorList.Length() - 1].SetTargetY2(targetY2)
	elevatorList[elevatorList.Length() - 1].SetDirectionByIndex(direction)
	elevatorList[elevatorList.Length() - 1].SetTargetDirection1(targetDirection1)
	elevatorList[elevatorList.Length() - 1].SetTargetDirection2(targetDirection2)
	elevatorList[elevatorList.Length() - 1].SetSpeed(speed)
End




Function SpawnPickUp:Void(x:Float, y:Float, animName:String)
	pickUpList = pickUpList.Resize(pickUpList.Length() + 1)
	pickUpList[pickUpList.Length() - 1] = New PickUpClass()
	
	pickUpList[pickUpList.Length() - 1].SetAnimation(animName)
	pickUpList[pickUpList.Length() - 1].SetX(x)
	pickUpList[pickUpList.Length() - 1].SetY(y)
End




Function SpawnExit:Void(x:Float, y:Float, animName:String)
	if exitDoor = Null
		exitDoor = New ExitClass()
	EndIf
	
	exitDoor.SetX(x)
	exitDoor.SetY(y)
	exitDoor.SetAnimation(animName)
End




Function SpawnDamsel:Void(x:Float, y:Float, animName:String)
	if damsel = Null
		damsel = New DamselClass()
	EndIf
	
	damsel.SetX(x)
	damsel.SetY(y)
	damsel.SetAnimation(animName)
End




Function ClearBaddieList:Void()
	Local index:Int
	
	
	For index = 0 to (baddieList.Length() - 1)
		if baddieList[index] <> Null
			baddieList[index] = Null
		EndIf
	Next
	
	baddieList = baddieList.Resize(0)
End




Function ClearElevatorList:Void()
	Local index:Int
	
	
	For index = 0 to (elevatorList.Length() - 1)
		if elevatorList[index] <> Null
			elevatorList[index] = Null
		EndIf
	Next
	
	elevatorList = elevatorList.Resize(0)
End




Function ClearPickUpList:Void()
	Local index:Int
	
	
	For index = 0 to (pickUpList.Length() - 1)
		if pickUpList[index] <> Null
			pickUpList[index] = Null
		EndIf
	Next
	
	pickUpList = pickUpList.Resize(0)
End




Function ClearDamsel:Void()
	if damsel <> Null
		damsel = Null
	EndIf
End




Function ClearPlayer:Void()
	if player <> Null
		player = Null
	EndIf
End




Function ClearExit:Void()
	if exitDoor <> Null
		exitDoor = Null
	EndIf
End




Function GrabPickups:Void(x:Float, y:Float)
	Local index:Int
	
	
	For index = 0 to (pickUpList.Length() - 1)
		if pickUpList[index] <> Null
			if x > (pickUpList[index].GetX() - tileWidth) And x < (pickUpList[index].GetX() + tileWidth)
				if y > (pickUpList[index].GetY() - tileHeight) And y < (pickUpList[index].GetY() + tileHeight)
					pickUpList[index] = Null
					score = (score + 50)
					PlaySound(soundPickUp, 1)
				EndIf
			EndIf
		EndIf
	Next
End




Function PickUpsRemain:Bool()
	Local index:Int
	Local flag:Bool = False
	
	
	For index = 0 to (pickUpList.Length() - 1)
		if pickUpList[index] <> Null
			flag = True
		EndIf
	Next
	
	Return flag
End




Function PlayerInput:Void()
	Local directionHoldMax:Float = 300
	
	
	if KeyTapped(KEY_UP) = True
		directionKeyHoldStart = Millisecs()
		player.SetTargetDirectionByIndex(1)
		'PlayerSlow()
	EndIf
	
	
	if KeyTapped(KEY_RIGHT) = True
		directionKeyHoldStart = Millisecs()
		player.SetTargetDirectionByIndex(2)
		'PlayerSlow()
	EndIf
	
	
	if KeyTapped(KEY_DOWN) = True
		directionKeyHoldStart = Millisecs()
		player.SetTargetDirectionByIndex(3)
		'PlayerSlow()
	EndIf
	
	
	if KeyTapped(KEY_LEFT) = True
		directionKeyHoldStart = Millisecs()
		player.SetTargetDirectionByIndex(4)
		'PlayerSlow()
	EndIf
	
	
	if player.GetDirection() <> player.GetTargetDirection()
		PlayerSlow()
	EndIf
	
	
	if player.GetDirection() = 1
		if KeyDown(KEY_UP) = True And Millisecs() >= (directionKeyHoldStart + directionHoldMax)
			PlayerFast()
		EndIf
	EndIf
	
	
	if player.GetDirection() = 2
		if KeyDown(KEY_RIGHT) = True And Millisecs() >= (directionKeyHoldStart + directionHoldMax)
		PlayerFast()
		EndIf
	EndIf
	
	
	if player.GetDirection() = 3
		if KeyDown(KEY_DOWN) = True And Millisecs() >= (directionKeyHoldStart + directionHoldMax)
			PlayerFast()
		EndIf
	EndIf
	
	
	if player.GetDirection() = 4
		if KeyDown(KEY_LEFT) = True And Millisecs() >= (directionKeyHoldStart + directionHoldMax)
			PlayerFast()
		EndIf
	EndIf
End




Function PlayerSlow:Void()
	player.SetSpeed(player.speedLow)
	player.SetAnimSpeed(player.animSpeedLow)
End




Function PlayerFast:Void()
	player.SetSpeed(player.speedHigh)
	player.SetAnimSpeed(player.animSpeedHigh)
End




'Function RestrictPlayer:Void()
'	Local gridRefX:Int = Floor(player.GetX() / tileWidth)
'	Local gridRefY:Int = Floor(player.GetY() / tileHeight)
'	
'	
'	if gridRefX < 0
'		gridRefX = 0
'	EndIf
'	
'	if gridRefY < 0
'		gridRefY = 0
'	EndIf
'	
'	
'	if player.GetY() <= 0
'		player.SetY(0)
'	EndIf
'	
'	if player.GetX() >= tileWidth * (tileMapColumnCount - 1)
'		player.SetX(tileWidth * (tileMapColumnCount - 1))
'	EndIf
'	
'	if player.GetY() >= tileHeight * (tileMapRowCount - 1)
'		player.SetY(tileHeight * (tileMapRowCount - 1))
'	EndIf
'	
'	if player.GetX() <= 0
'		player.SetX(0)
'	EndIf
'	
'	
'	if exitDoor <> Null
'		if PlayerTouchingExit() = True
'			if player.GetDirection() = 1 And player.GetY() > exitDoor.GetY()
'				player.SetY(exitDoor.GetY() + (tileHeight * 2))
'			EndIf
'			
'			if player.GetDirection() = 2 And player.GetX() < exitDoor.GetX()
'				player.SetX(exitDoor.GetX() - tileWidth)
'			EndIf
'			
'			if player.GetDirection() = 3 And player.GetY() < exitDoor.GetY()
'				player.SetY(exitDoor.GetY() - tileHeight)
'			EndIf
'			
'			if player.GetDirection() = 4 And player.GetX() > exitDoor.GetX()
'				player.SetX(exitDoor.GetX() + (tileWidth * 2))
'			EndIf
'		EndIf
'	EndIf
'	
'	
'	if player.GetDirection() = 1
'		if GetTileInfo(gridRefX, gridRefY) > 0
'			player.SetY(tileHeight * (gridRefY + 1))
'			player.canMove = False
'		Else
'			player.canMove = True
'		EndIf
'	EndIf
'	
'	if player.GetDirection() = 3 and gridRefY < (tileMapRowCount - 1)
'		if GetTileInfo(gridRefX, gridRefY + 1) > 0
'			player.SetY(tileHeight * gridRefY)
'			player.canMove = False
'		Else
'			player.canMove = True
'		EndIf
'	EndIf
'	
'	if player.GetDirection() = 2 And gridRefX < (tileMapColumnCount - 1)
'		if GetTileInfo(gridRefX + 1, gridRefY) > 0
'			player.SetX(tileWidth * gridRefX)
'			player.canMove = False
'		Else
'			player.canMove = True
'		EndIf
'	EndIf
'	
'	if player.GetDirection() = 4
'		if GetTileInfo(gridRefX, gridRefY) > 0
'			player.SetX(tileWidth * (gridRefX + 1))
'			player.canMove = False
'		Else
'			player.canMove = True
'		EndIf
'	EndIf
'End




Function RestrictPlayer:Void()
	Local gridRefX:Int = Floor(player.GetX() / tileWidth)
	Local gridRefY:Int = Floor(player.GetY() / tileHeight)
	
	
	if gridRefX < 0
		gridRefX = 0
		player.SetX(gridRefX)
	EndIf
	
	if (gridRefX + 1) >= tileMapColumnCount
		gridRefX = (tileMapColumnCount - 1)
		player.SetX(tileWidth * gridRefX)
	EndIf
	
	if gridRefY < 0
		gridRefY = 0
		player.SetY(gridRefY)
	EndIf
	
	if (gridRefY + 1) >= tileMapRowCount
		gridRefY = (tileMapRowCount - 1)
		player.SetY(tileHeight * gridRefY)
	EndIf
	
	
	if player.GetDirection() = 1 And gridRefY >= 0
		if GetTileInfo(gridRefX, gridRefY) <> 0
			player.SetY(tileHeight * (gridRefY + 1))
		EndIf
	EndIf
	
	if player.GetDirection() = 2 And gridRefX < (tileMapColumnCount - 1)
		if GetTileInfo(gridRefX + 1, gridRefY) <> 0
			player.SetX(tileWidth * gridRefX)
		EndIf
	EndIf
	
	if player.GetDirection() = 3 And gridRefY < (tileMapRowCount - 1)
		if GetTileInfo(gridRefX, gridRefY + 1) <> 0
			player.SetY(tileHeight * gridRefY)
		EndIf
	EndIf
	
	if player.GetDirection() = 4 And gridRefX >= 0
		if GetTileInfo(gridRefX, gridRefY) <> 0
			player.SetX(tileWidth * (gridRefX + 1))
		EndIf
	EndIf
	
	
	if exitDoor <> Null And exitDoor.IsOpen() = False
		if player.GetX() > (exitDoor.GetX() - tileWidth) And player.GetX() < (exitDoor.GetX() + (tileWidth * 2))
			if player.GetDirection() = 1
				if player.GetY() > (exitDoor.GetY() - tileHeight) And player.GetY() < (exitDoor.GetY() + (tileHeight * 2))
					player.SetY(exitDoor.GetY() + (tileHeight * 2))
				EndIf
			EndIf
			
			if player.GetDirection() = 3
				if player.GetY() > (exitDoor.GetY() - tileHeight) And player.GetY() < (exitDoor.GetY() + (tileHeight * 2))
					player.SetY(exitDoor.GetY() - tileHeight)
				EndIf
			EndIf
		EndIf
		
		if player.GetY() > (exitDoor.GetY() - tileHeight) And player.GetY() < (exitDoor.GetY() + (tileHeight * 2))
			if player.GetDirection() = 2
				if player.GetX() > (exitDoor.GetX() - tileWidth) And player.GetX() < (exitDoor.GetX() + (tileWidth * 2))
					player.SetX(exitDoor.GetX() - tileWidth)
				EndIf
			EndIf
			
			if player.GetDirection() = 4
				if player.GetX() > (exitDoor.GetX() - tileWidth) And player.GetX() < (exitDoor.GetX() + (tileWidth * 2))
					player.SetX(exitDoor.GetX() + (tileWidth * 2))
				EndIf
			EndIf
		EndIf
	EndIf
	
	
	if player.GetDirection() = 1 or player.GetDirection() = 3
		if player.GetTargetDirection() = 1 or player.GetTargetDirection() = 3
			player.SetDirectionByIndex(player.GetTargetDirection())
		EndIf
	EndIf
	
	if player.GetDirection() = 2 or player.GetDirection() = 4
		if player.GetTargetDirection() = 2 or player.GetTargetDirection() = 4
			player.SetDirectionByIndex(player.GetTargetDirection())
		EndIf
	EndIf
	
	
	if player.GetDirection() = 1 or player.GetDirection() = 3
		if player.GetTargetDirection() = 2
			if player.GetY() > (tileHeight * (gridRefY + 1)) - (tileHeight + 3) And player.GetY() < (tileHeight * (gridRefY + 1)) - (tileHeight - 3)
				player.SetY(tileHeight * gridRefY)
			EndIf
		EndIf
		
		if player.GetTargetDirection() = 4
			if player.GetY() < (tileHeight * (gridRefY + 1)) - (tileHeight - 3) And player.GetY() > (tileHeight * (gridRefY + 1)) - (tileHeight + 3)
				player.SetY(tileHeight * gridRefY)
			EndIf
		EndIf
		
		if player.InlineWithTileMapRow() = True
			player.SetDirectionByIndex(player.GetTargetDirection())
		EndIf
	EndIf
	
	
	if player.GetDirection() = 2 or player.GetDirection() = 4
		if player.GetTargetDirection() = 1
			if player.GetX() < (tileWidth * (gridRefX + 1)) - (tileWidth - 3) And player.GetX() > (tileWidth * (gridRefX + 1)) - (tileWidth + 3)
				player.SetX(tileWidth * gridRefX)
			EndIf
		EndIf
		
		if player.GetTargetDirection() = 3
			if player.GetX() > (tileWidth * (gridRefX + 1)) - (tileWidth + 3) And player.GetX() < (tileWidth * (gridRefX + 1)) - (tileWidth - 3)
				player.SetX(tileWidth * gridRefX)
			EndIf
		EndIf
		
		if player.InlineWithTileMapColumn() = True
			player.SetDirectionByIndex(player.GetTargetDirection())
		EndIf
	EndIf
End




Function PlayerTouchingExit:Bool()
	Local flag:Bool = False
	
	
	if player <> Null And exitDoor <> Null
		if player.GetX() > (exitDoor.GetX() - tileWidth) And player.GetX() < (exitDoor.GetX() + (tileWidth * 2))
			if player.GetY() > (exitDoor.GetY() - tileHeight) And player.GetY() < (exitDoor.GetY() + (tileHeight * 2))
				flag = True
			EndIf
		EndIf
	EndIf
	
	Return flag
End




Function PlayerTouchingDamsel:Bool()
	Local flag:Bool = False
	
	
	if player <> Null And damsel <> Null
		if player.GetX() > (damsel.GetX() - tileWidth) And player.GetX() < (damsel.GetX() + (tileWidth * 3))
			if player.GetY() > (damsel.GetY() - tileHeight) And player.GetY() < (damsel.GetY() + (tileHeight * 6))
				flag = True
			EndIf
		EndIf
	EndIf
	
	Return flag
End




Function PlayerTouchingBaddie:Bool()
	Local index:Int
	Local flag:Bool = False
	Local playerFrame:Int
	Local baddieFrame:Int
	
	
	playerFrame = spriteAnimTable[player.animTable[player.GetDirection()]].GetCurrentFrameContent()
	
	For index = 0 to (baddieList.Length() - 1)
		if baddieList[index] <> Null
			baddieFrame = spriteAnimTable[baddieList[index].anim].GetCurrentFrameContent()
			
			if SpriteFramesCollide(playerFrame, player.GetX(), player.GetY(), baddieFrame, baddieList[index].GetX(), baddieList[index].GetY()) = True
				flag = True
			EndIf
		EndIf
	Next
	
	Return flag
End




Function PlayerTouchingElevator:Bool()
	Local index:Int
	Local flag:Bool = False
	Local playerFrame:Int
	Local elevatorFrame:Int
	
	
	playerFrame = spriteAnimTable[player.animTable[player.GetDirection()]].GetCurrentFrameContent()
	
	For index = 0 to (elevatorList.Length() - 1)
		if elevatorList[index] <> Null
			elevatorFrame = spriteAnimTable[elevatorList[index].anim].GetCurrentFrameContent()
			
			if SpriteFramesCollide(playerFrame, player.GetX(), player.GetY(), elevatorFrame, elevatorList[index].GetX(), elevatorList[index].GetY()) = True
				flag = True
			EndIf
		EndIf
	Next
	
	Return flag
End




Function PlayerDie:Void()
	playerDieing = 4
	playerDieTimer = New TimerClass(100)
	titleMusicState = 1
	lives = (lives - 1)
	
	StopMusic()
	SetGameState(PLAYER_DIE)
	
	
	if ChannelState(0) = 0
		PlaySound(soundPlayerDie, 0)
	Endif
End




Function UpdatePlayerDie:Void()
	if playerDieTimer <> Null And playerDieTimer.GetTrigger() = True
		playerDieing = (playerDieing - 1)
	EndIf
	
	if playerDieing <= 0 And ChannelState(0) = 0
		RestartLevel()
		SetGameState(GAME)
	EndIf
End




Function PlayerDead:Bool()
	Local flag:Bool = False
	
	
	if playerDieing <= 0 And LivesRemain() = False
		flag = True
	EndIf
	
	Return flag
End




Function DrawPlayerDie:Void()
	Local colourIndex:Int
	
	
	if playerDieing > 0
		colourIndex = (playerDieColorBurst.Length() - 2) - playerDieing
		Cls playerDieColorBurst[colourIndex], playerDieColorBurst[colourIndex + 1], playerDieColorBurst[colourIndex + 2]
	EndIf
	
	DrawTileMap(0, 0)
	
	
	if gridOn = 1	
		SetColor(100, 100, 100)
		DrawGrid(0, 0)
	EndIf
	
	
	DrawElevators()
	DrawBaddies()
	DrawPickUps()
	
	
	if exitDoor <> Null
		exitDoor.Draw()
	EndIf
	
	if damsel <> Null
		damsel.Draw()
	EndIf
	
	if player <> Null And playerHide = False
		player.Draw()
	EndIf
	
	
	DrawLives()
	DrawInGameInfo()
	DrawLevelName()
End




Function Mid:String(sString:String, startPos:Int, charCount:Int)
	Local outAsc:String
	Local i:int
	Local startPosFlag:Bool
	Local charCountFlag:Bool
	
	
	If startPos > 0 And startPos <= sString.Length()
		startPosFlag = True
	Else
		startPosFlag = False
	EndIf
	
	If charCount > 0 And charCount <= sString.Length() - (startPos - 1)
		charCountFlag = True
	Else
		charCountFlag = False
	EndIf
	
	If startPosFlag = True And charCountFlag = True
		outAsc = sString[startPos - 1 .. startPos + charCount - 1]
	EndIf
	
	If startPosFlag = False
		outAsc = "Misuse of custom Mid function!"
	EndIf
	
	If charCountFlag = False
		outAsc = "Misuse of custom Mid function!"
	EndIf
	
	If startPosFlag = False And charCountFlag = False
		outAsc = "Misuse of custom Mid function!"
	EndIf
	
	Return outAsc
End




Function ReadFile:FileLoaderClass(fileName:String)
	Return New FileLoaderClass(fileName)
End




Function CloseFile:Void(rFile:FileLoaderClass)
	rFile.Empty()
	rFile = Null
End




Function FirstInString:String(sString:String)
	Local endOfFirst:Int
	Local outString:String
	
	
	endOfFirst = sString.Find(" ")
	outString = sString[0 .. endOfFirst]
	
	Return outString
End




Function HexToInt:Int(hexIn:String)
	Local hexRefTable:String
	Local tokenValue:Int
	Local column:Int
	Local intOut:Int
	
	
	hexIn = hexIn.ToUpper()
	hexRefTable = "0123456789ABCDEF"
	
	For column = 0 To hexIn.Length() - 1
		tokenValue = hexRefTable.Find(hexIn[column .. column + 1])
		intOut = (intOut * 16) + tokenValue
	Next
	
	Return intOut
End




Function AddMessage:Void(message:String)
	messageStack = messageStack.Resize(messageStack.Length() + 1)
	messageStack[messageStack.Length() - 1] = message
End




Function ShowMessages:Void(x:Int, y:Int)
	Local index:Int
	
	
	For index = 0 to (messageStack.Length() - 1)
		DrawText(messageStack[index], x, (index * 15) + y)
	Next
End




Function ClearMessages:Void()
	Local index:Int
	
	
	For index = 0 to (messageStack.Length() - 1)
		messageStack[index] = ""
	Next
	
	messageStack = messageStack.Resize(0)
End




Function GetScaleX:Float(nativeX:Float)
	Return (nativeX / nativeWidth) * DeviceWidth()
End




Function GetScaleY:Float(nativeY:Float)
	Return (nativeY / nativeHeight) * DeviceHeight()
End




Function GetScaleX:Float()
	Return (DeviceWidth() / nativeWidth)
End




Function GetScaleY:Float()
	Return (DeviceHeight() / nativeHeight)
End




Function SetNativeValues:Void()
	SetNativeScreenSize()
	SetMapTileSizes()
End




Function SetNativeScreenSize:Void()
	nativeWidth = 640
	nativeHeight = 480
End




Function SetMapTileSizes:Void()
	tileWidth = 40
	tileHeight = 32
End




Function SetGameState:Void(gState:Int)
	gameState = gState
End




Function GetGameState:Int()
	Return gameState
End




Function Initiate:Void()
	
End




Function NewGame:Void()
	congratsMessageState = 0
	currentLevel = 1
	lives = 5
	score = 0
	bonusScore = 1000
	playerDieing = 0
	titleMusicState = 2
	playerHide = False
	gamePaused = 0
	pauseScroll = 0
	
	StopMusic()
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearPlayer()
	ClearExit()
	ClearDamsel()
	SetGameState(GAME)
	LoadLevel(currentLevel)
	LoadPickUps(currentLevel)
End




Function ExitGame:Void()
	lives = 0
	PlayerDie()
End




Function PreviousLevel:Void()
	currentLevel = (currentLevel - 1)
	playerDieing = 0
	bonusScore = 1000
	titleMusicState = 2
	
	if currentLevel < 1
		currentLevel = 41
	EndIf
	
	StopChannel(0)
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearPlayer()
	ClearExit()
	ClearDamsel()
	LoadLevel(currentLevel)
	LoadPickUps(currentLevel)
End




Function NextLevel:Void()
	currentLevel = (currentLevel + 1)
	playerDieing = 0
	bonusScore = 1000
	titleMusicState = 2
	
	
	if currentLevel > 41
		currentLevel = 1
	EndIf
	
	
	if currentLevel = 20
		lives = 5
	EndIf
	
	
	StopChannel(0)
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearPlayer()
	ClearExit()
	ClearDamsel()
	LoadLevel(currentLevel)
	LoadPickUps(currentLevel)
End




Function RandomDemoLevel:Void()
	Local previousLevel:Int = currentLevel
	
	
	currentLevel = Floor(Rnd() * 40)
	
	if currentLevel = previousLevel
		currentLevel = Floor(Rnd() * 40)
	EndIf
	
	
	if currentLevel < 1
		currentLevel = 1
	EndIf
	
	
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearExit()
	ClearDamsel()
	LoadLevel(currentLevel)
	LoadPickUps(currentLevel)
End




Function TitleLevel:Void()
	currentLevel = 0
	
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearExit()
	ClearDamsel()
	LoadLevel(currentLevel)
End




Function RestartLevel:Void()
	ClearBaddieList()
	ClearElevatorList()
	ClearPlayer()
	ClearExit()
	ClearDamsel()
	LoadLevel(currentLevel)
End




Function EndGame:Void()
	titleScrollProgress = 590.00
	titleMusicState = 1
	
	ClearBaddieList()
	ClearElevatorList()
	ClearPickUpList()
	ClearPlayer()
	ClearExit()
	ClearDamsel()
	StopMusic()
	StartTitle()
End




Function StartBonus:Void()
	playerHide = True
	PlaySound(soundBonus, 0)
	SetGameState(BONUS)
End




Function UpdateBonus:Void()
	Local multiplier:Int = 10
	
	
	bonusScore = (bonusScore - multiplier)
	score = (score + multiplier)
	
	if bonusScore <= 0
		bonusScore = 1000
		EndBonus()
	EndIf
End




Function DrawBonus:Void()
	DrawGame()
End




Function EndBonus:Void()
	SetGameState(GAME)
	NextLevel()
	playerHide = False
End




Function StartCongrats:Void()
	PlayCongratsMusic()
End




Function PlayCongratsMusic:Void()
	SetGameState(CONGRATS_MUSIC)
	PlayMusic("audio/congratulations" + fileExtAudio, 0)
End




Function UpdateCongrats:Void()
	if MusicState() = 0 And GetGameState() = CONGRATS_MUSIC
		StartFinalBonus()
	EndIf
	
	if GetGameState() = CONGRATS_MESSAGE
		if congratsTimer = Null
			congratsTimer = New TimerClass(3000)
		EndIf
		
		UpdateCongratsMessage()
	EndIf
End




Function UpdateCongratsMessage:Void()
	if congratsTimer <> Null And congratsTimer.GetTrigger() = True
		congratsMessageState = (congratsMessageState + 1)
		
		if congratsMessageState >= 3
			congratsTimer = Null
			EndCongrats()
		EndIf
	EndIf
End




Function DrawCongrats:Void()
	Local yBase:Float = 128
	
	
	Cls
	
	if congratsMessageState = 0 OR congratsMessageState = 1
		DrawString(2, "WELL DONE-", 0, yBase + 60, "alignCentre")
	EndIf
	
	if congratsMessageState = 1
		DrawString(5, "MONTY", 0, yBase + 100, "alignCentre")
		DrawString(5, "HAS SAVED THE DAMSEL", 0, yBase + 120, "alignCentre")
	EndIf
	
	if congratsMessageState = 2
		DrawString(3, "BET YOU", 0, yBase + 80, "alignCentre")
		DrawString(3, "CAN'T DO IT AGAIN!", 0, yBase + 100, "alignCentre")
	EndIf
End




Function EndCongrats:Void()
	EndGame()
End




Function StartFinalBonus:Void()
	PlaySound(soundBonus, 0)
	SetGameState(FINAL_BONUS)
End




Function UpdateFinalBonus:Void()
	Local multiplier:Int = 10
	
	
	bonusScore = (bonusScore - multiplier)
	score = (score + multiplier)
	
	if bonusScore <= 0
		bonusScore = 1000
		EndFinalBonus()
	EndIf
End




Function DrawFinalBonus:Void()
	DrawGame()
End




Function EndFinalBonus:Void()
'	EndGame()
	SaveHighScore()
	SetGameState(CONGRATS_MESSAGE)
'	StartTitle()
End




Function StartGameOver:Void()	
	gameOverMessageProgress = 0
	SetGameState(GAME_OVER)
End




Function UpdateGameOver:Void()
	if gameOverMessageTicker.GetTrigger() = True
		if gameOverMessageProgress < 9
			if ChannelState(1) = 1
				StopChannel(1)
			EndIf
			
			PlaySound(soundBeep, 1)
			gameOverMessageProgress = (gameOverMessageProgress + 1)
			pauseTimer.SetDelay(1000)
		EndIf
	EndIf
	
	if gameOverMessageProgress > 8 And pauseTimer.GetTrigger() = True
		EndGameOver()
	EndIf
End




Function DrawGameOver:Void()
	Cls 0, 0, 0
	DrawImage(gameOverMessageGFX.GrabImage(0, 0, (gameOverMessageProgress + 1) * 40, 32), ((nativeWidth / 2) - 180) * GetScaleX(), ((nativeHeight / 2) - 16) * GetScaleY())
End




Function EndGameOver:Void()
	StartTruckChase()
End




Function StartTruckChase:Void()
	ClearBaddieList()
	ClearPlayer()
	LoadLevel(100)
	SetGameState(TRUCK_CHASE)
	PlaySound(soundTruckChase, 0)
End




Function UpdateTruckChase:Void()
	Local index:Int
	
	
	if player <> null
		player.Update()
		
		if player.GetX() <= 0
			player.SetX(0)
		EndIf
	EndIf
	
	UpdateBaddies()
	UpdateSpriteAnims()
	
	if baddieList[0].GetX() <= 0
		EndTruckChase()
	EndIf
End




Function DrawTruchChase:Void()
	Cls 0, 0, 0
	
	DrawTileMap(0, 0)
	
	if player <> Null
		player.Draw()
	Endif
	
	DrawBaddies()
End




Function EndTruckChase:Void()
	ClearPlayer()
	ClearBaddieList()
	StartTitle()
End




Function UpdateGame:Void()
	Local i:Int
	
	
	if PickUpsRemain() = False And exitDoor <> Null
		exitDoor.SetAnimation("ExitOpen")
	EndIf
	
	
	if player <> Null
		PlayerInput()
		player.Update()
		RestrictPlayer()
	EndIf
	
	
	if PlayerTouchingExit() = True And exitDoor.IsOpen() = True
		StopMusic()
		StartBonus()
	EndIf
	
	
	if PlayerTouchingDamsel() = True
		StartCongrats()
	EndIf
	
	
	if PlayerTouchingBaddie() = True or PlayerTouchingElevator() = True
		PlayerDie()
	EndIf
	
	
	UpdateBaddies()
	UpdateElevators()
	UpdateSpriteAnims()
	GrabPickups(player.GetX(), player.GetY())
	
	
	if bonusTicker.GetTrigger() = True
		bonusScore = (bonusScore - 10)
		
		if bonusScore < 0
			bonusScore = 1000
			PlayerDie()
		EndIf
	EndIf
	
	
	if DEBUG_ENABLED = True
		if KeyTapped(KEY_X) = True
			NextLevel()
			
			if KeyDown(KEY_SHIFT) = True
				For i = 1 to 8
					NextLevel()
				Next
			EndIf
		EndIf
		
		if KeyTapped(KEY_Z) = True
			PreviousLevel()
			
			if KeyDown(KEY_SHIFT) = True
				For i = 1 to 8
					PreviousLevel()
				Next
			EndIf
		EndIf
	EndIf
	
	
	if KeyTapped(KEY_ESCAPE) = True
		ExitGame()
	EndIf
	
	
	if PlayerDead() = True
		StopMusic()
		EndGame()
		StartGameOver()
		SaveHighScore()
	EndIf
End




Function DrawGame:Void()
	Local colourIndex:Int
	
	
	if playerDieing <= 0
		Cls 0, 0, 0
	EndIf
	
	
	DrawTileMap(0, 0)
	
	if gridOn = 1	
		SetColor(100, 100, 100)
		DrawGrid(0, 0)
	EndIf
	
	
	DrawElevators()
	DrawBaddies()
	DrawPickUps()
	
	
	if exitDoor <> Null
		exitDoor.Draw()
	EndIf
	
	if damsel <> Null
		damsel.Draw()
	EndIf
	
	if player <> Null And playerHide = False
		player.Draw()
	EndIf
	
	
	DrawLives()
	DrawInGameInfo()
	DrawLevelName()
	DrawInGameInfo()
'	DrawText(MusicState(), 0, 480)
	
	ShowMessages(0, 340)
	ClearMessages()
End




Function DrawPause:Void()
	Local x:Float
	Local y:Float
	
	
	x = 320 - (pauseGFX.Width() / 2)
	y = 240 - (pauseGFX.Height() / 4)
	
	DrawImageRect(pauseGFX, x * GetScaleX(), y * GetScaleY(), 0, pauseScroll, pauseGFX.Width(), pauseGFX.Height() / 2, 0, GetScaleX(), GetScaleY())
End




Function UpdatePause:Void()
	Local pixelsPerSecond:Float = (100 * delta)
	
	
	pauseScroll = (pauseScroll - (pixelsPerSecond * 0.5))
	
	if pauseScroll < 0
		pauseScroll = (pauseGFX.Height() / 2)
	EndIf
End




Function DrawInGameInfo:Void()
	Local x:Float = 0
	Local y:Float = (tileHeight * GetScaleY()) * (tileMapRowCount + 0)
	
	
	SetFontScale(0.75, 0.75)
	DrawString(1, "SCORE", (tileWidth * GetScaleX()) * 6, y)
	DrawString(1, "HIGH", (tileWidth * GetScaleX()) * 10, y)
	DrawString(1, "BONUS", (tileWidth * GetScaleX()) * 14, y)
	
	DrawString(2, score, (tileWidth * GetScaleX()) * 6, y + (tileHeight / 2))
	DrawString(2, highScore, (tileWidth * GetScaleX()) * 10, y + (tileHeight / 2))
	DrawString(2, bonusScore, (tileWidth * GetScaleX()) * 14, y + (tileHeight / 2))
End




Function DrawLives:Void()
	Local x:Float = 0
	Local y:Float = (tileHeight * GetScaleY()) * (tileMapRowCount + 0)
	Local index:Int
	
	
	For index = 0 to (lives - 1)
		DrawSpriteFrame(5, (tileWidth * index) + (x * GetScaleX()), y * GetScaleY(), False, False)
	Next
End




Function LivesRemain:Bool()
	Local flag:Bool = False
	
	
	if lives >= 1
		flag = True
	EndIf
	
	Return flag
End




Function DrawLevelName:Void()
	SetFontScale(0.75, 0.75)
	DrawString(3, GetLevelTitle(), nativeWidth / 2, (tileMapRowCount + 2) * tileHeight, "alignCentre")
	
	if GetGameState() = DEMO
		SetFontScale(0.45, 0.5)
		DrawString(6, "Arrow keys change direction. Hold arrow key in current direction to run.", 0, ((tileMapRowCount + 3) * tileHeight) + (tileHeight / 2), "alignCentre")
		DrawString(6, "P pauses game. Esc quits the game.", 0, ((tileMapRowCount + 3) * tileHeight) + tileHeight, "alignCentre")
	EndIf
End




Function UpdateTitle:Void()
	UpdateTitleScroll()
	UpdateTitleLevel()
	
	if LinkClicked("Contact BulbzGamez") = True
		OpenUrl("mailto:stephen.ancell@ntlworld.com")
	EndIf
End




Function UpdateTitleLevel:Void()
	UpdateBaddies()
	UpdateElevators()
	UpdateSpriteAnims()
End




Function UpdateTitleScroll:Void()
	Local pixelsPerSecond:Float = (100 * delta)
	
	
	titleScrollProgress = (titleScrollProgress - 3.00)
	
	if ScrollEnded() = True
		titleScrollProgress = 590.00
		
		DeleteAllLinks()
		StartDemo()
	EndIf
End




Function ScrollEnded:Bool()
	Local flag:Bool = False
	
	
	if titleScrollProgress < 120 - (GetStringLength(titleScroll) * (17 * 0.65))
		flag = True
	EndIf
	
	Return flag
End




Function DrawTitle:Void()
	Cls 0, 0, 0
	
	SetFontScale(0.65, 0.70)
	DrawString(2, titleScroll, titleScrollProgress, 340)
	
	DrawImage(titleGFX1, 0, 0, 0, GetScaleX(), GetScaleY())
	
	SetFontScale(0.65, 0.80)
	
	if DEBUG_ENABLED = True
		DrawString(2, "(Currently work in progress!!!)", 0, 80, "alignCentre")
	EndIf
	
	DrawString(3, "A remake of the 1984 Amstrad CPC classic", 100, 210)
	
	SetFontScale(0.60, 0.65)
	DrawString(2, "Coding by: Steve Ancell", 110, 240)
	DrawString(4, "GFX and Audio: Ripped from the original", 120, 260)
	DrawString(4, "And the Bulbz!... Those are mine!!!", 130, 280)
	
	SetFontScale(1.20, 1.25)
	DrawString(startButtonState, "Click Here To Play!", 0, 160, "alignCentre")
	
	DrawTitleLevel()
	DrawLinks()
End




Function DrawTitleLevel:Void()
	DrawTileMap(0, 0)
	DrawElevators()
	DrawBaddies()
	DrawPickUps()
	
	if exitDoor <> Null
		exitDoor.Draw()
	EndIf
	
	if damsel <> Null
		damsel.Draw()
	EndIf
End




Function PlayTitleMusic:Void()
	if MusicState() = 0 And musicOn = 1
		if titleMusicState = 1 And ChannelState(0) = 0
			PlayMusic("audio/musicintro" + fileExtAudio, 0)
		EndIf
		
		if titleMusicState = 2 And ChannelState(0) = 0
			PlayMusic("audio/musicmain" + fileExtAudio, 0)
		EndIf
		
		titleMusicState = (titleMusicState ~ 3)
	EndIf
	
	
	if musicOn = 0
		if GetGameState() = GAME or GetGameState() = TITLE
			StopMusic()
		EndIf
	EndIf
End




Function StartTitle:Void()
	CreateLink("Contact BulbzGamez", 320, 440, 0.65, 0.70, "alignCentre")
	SetGameState(TITLE)
	TitleLevel()
End




Function StartDemo:Void()
	demoProgress = 0
	demoTimer = New TimerClass(5000)
	
	SetGameState(DEMO)
	RandomDemoLevel()
End




Function UpdateDemo:Void()
	if demoTimer.GetTrigger() = True
		RandomDemoLevel()
		demoProgress = (demoProgress + 1)
	EndIf
	
	if demoProgress >= 4 or KeyTapped(KEY_ESCAPE) = True
		EndDemo()
	EndIf
	
	UpdateBaddies()
	UpdateElevators()
	UpdateSpriteAnims()
End




Function DrawDemo:Void()
	Cls 0, 0, 0
	
	DrawTileMap(0, 0)
	DrawElevators()
	DrawBaddies()
	DrawPickUps()
	
	if exitDoor <> Null
		exitDoor.Draw()
	EndIf
	
	if damsel <> Null
		damsel.Draw()
	EndIf
	
	DrawLevelName()
End




Function EndDemo:Void()
	demoTimer = Null
	
	titleScrollProgress = 590.00
	StartTitle()
End




Function KeyTapped:Bool(keyCode:Int)
	Local flag:Bool = False
	
	
	if keytappedRegister[keyCode] = 0
		if KeyDown(keyCode) = True
			keytappedRegister[keyCode] = 1
			flag = True
		EndIf
	EndIf
	
	if KeyDown(keyCode) = False
		keytappedRegister[keyCode] = 0
	EndIf
	
	Return flag
End




Function CreateLink:Void(address:String, x:Float = 0, y:Float = 0, scaleX:Float = 1.00, scaleY:Float = 1.00, directives:String = "")
	addressList = addressList.Resize(addressList.Length() + 1)
	addressList[addressList.Length() - 1] = New LinkClass(address, scaleX, scaleY, directives)
	addressList[addressList.Length() - 1].SetPosition(x, y)
End




Function LinkClicked:Bool(address:String)
	Local flag:Bool = false
	Local index:Int
	
	For index = 0 to (addressList.Length() - 1)
		if addressList[index] <> Null
			if addressList[index].GetAddress() = address
				if addressList[index].Clicked() = True
					flag = True
				EndIf
			EndIf
		EndIf
	Next
	
	Return flag
End




Function DrawLinks:Void()
	Local index:Int
	
	For index = 0 to (addressList.Length() - 1)
		if addressList[index] <> Null
			addressList[index].Draw()
		EndIf
	Next
End




Function DeleteLink:Void(address:String)
	Local index:Int
	
	For index = 0 to (addressList.Length() - 1)
		if addressList[index] <> Null
			if addressList[index].GetAddress() = address
				addressList[index] = Null
			EndIf
		Endif
	Next
End




Function DeleteAllLinks:Void()
	Local index:Int
	
	For index = 0 to (addressList.Length() - 1)
		if addressList[index] <> Null
			DeleteLink(addressList[index].GetAddress())
		EndIf
	Next
	
	addressList = addressList.Resize(0)
End




Function SetFontScaleXY:Void(scaleX:Float, scaleY:Float)
	fontScaleX = scaleX
	fontScaleY = scaleY
End




Function DrawString:Void(fontIndex:Int, stringData:String, x:Float, y:Float, directives:String = "")
	Local chars:Int[] = stringData.ToChars()
	Local index:Int
	
	
	if directives.Contains("centreX")
		x = x - ((GetStringWidth(stringData) / 2) * fontScaleX)
	EndIf
		
	if directives.Contains("centreY")
		
	EndIf
	
	if directives.Contains("alignCentre")
		x = (nativeWidth / 2) - ((GetStringWidth(stringData) / 2) * fontScaleX)
	EndIf
	
	if directives.Contains("alignMiddle")
		
	EndIf
	
	if directives.Contains("alignRight")
		
	EndIf
	
	if directives.Contains("alignBottom")
		
	EndIf
	
	if directives.Contains("highlight")
		SetColor(140, 148, 155)
		DrawRect(x * GetScaleX(), y * GetScaleY(), GetStringWidth(stringData) * (GetScaleX() * fontScaleX), GetStringHeight() * (GetScaleY() * fontScaleY))
	EndIf
		
	For index = 0 to (GetStringLength(stringData) - 1)
		DrawChar(fontIndex, chars[index], x + (index * (17 * fontScaleX)), y)
	Next
End




Function DrawChar:Void(fontIndex:Int, charIndex:Int, x:Float, y:Float)
	if x >= (0 - 17) And x <= nativeWidth
		if y >= (0 - 26) And y <= nativeHeight
			DrawImage(imageFont.GrabImage((charIndex - 32) * 17, (fontIndex - 1) * 26, 17, 26), x * GetScaleX(), y * GetScaleY(), 0, GetScaleX() * fontScaleX, GetScaleY() * fontScaleY)
		EndIf
	EndIf
End




Function HoverString:Bool(stringData:String, x:Float, y:Float, directives:String = "")
	Local chars:Int[] = stringData.ToChars()
	Local flag:Bool = False
	Local index:Int
	
	
	if directives.Contains("centreX")
		x = x - ((GetStringWidth(stringData) / 2) * fontScaleX)
	EndIf
		
	if directives.Contains("centreY")
		
	EndIf
	
	if directives.Contains("alignCentre")
		x = (nativeWidth / 2) - ((GetStringWidth(stringData) / 2) * fontScaleX)
	EndIf
	
	if directives.Contains("alignMiddle")
		
	EndIf
	
	if directives.Contains("alignRight")
		
	EndIf
	
	if directives.Contains("alignBottom")
		
	EndIf
	
	
	if MouseX() >= (x * GetScaleX()) And MouseX() <= (x * GetScaleX()) + (GetStringWidth(stringData) * (GetScaleX() * fontScaleX))
		if MouseY() >= (y * GetScaleY()) And MouseY() <= (y * GetScaleY()) + (GetStringHeight() * (GetScaleY() * fontScaleY))
			flag = True
		EndIf
	EndIf
	
	Return flag
End




Function GetStringLength:Int(stringData:String)
	Local chars:Int[] = stringData.ToChars()
	
	
	Return chars.Length()
End




Function GetStringWidth:Float(stringData:String)
	Return (stringData.Length() * 17)
End




Function GetStringHeight:Float(stringData:String = "")
	Return 26
End




Function SetFontScale:Void(scaleX:Float, scaleY:Float)
	fontScaleX = scaleX
	fontScaleY = scaleY
End




Function ShowVars:Void()
'DrawText(fontScaleX, 0, 20)
'DrawText(fontScaleY, 0, 35)
'DrawText(GetScaleX(), 0, 50)
'DrawText(GetScaleY(), 0, 65)
'
'messageStack:String[]
'addressList:LinkClass[]
'keytappedRegister:Int[127]
'
'tileSheet:Image
'tileFrame:FrameClass[]
'tileMapColumnCount:Int
'tileMapRowCount:Int
'tileWidth:Float
'tileHeight:Float
'tileMap:Int[]
'
'
'spriteSheet:Image
'spriteFrame:FrameClass[]
'spriteAnimTable:AnimationClass[]
'spriteCollisionData:Int[]
'
'
'titleGFX1:Image
'imageFont:Image
'titleScroll:String
'titleScrollProgress:Float
'
'
'demoProgress:Int = 0
'demoTimer:TimerClass
'
'
'titleMusicState:Int = 1
'musicOn:Int = 1
'
'
'soundPickUp:Sound
'soundPlayerDie:Sound
'
'
'currentLevel:Int
'levelTitle:String
'
'
'player:PlayerClass
'playerHide:Bool
'directionKeyHoldStart:Float
'playerDieing:Int
'playerDieTimer:TimerClass
'playerDieColorBurst:Int[] = [255, 255, 0, 0, 255, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255]
'
'
'baddieList:BaddieClass[]
'elevatorList:ElevatorClass[]
'pickUpList:PickUpClass[]
'exitDoor:ExitClass
'damsel:DamselClass
'
'
'Const LOADING:Int = 0
'Const TITLE:Int = 1
'Const GAME:Int = 2
'Const PAUSE:Int = 3
'Const DEMO:Int = 4
'Const CONGRATS:Int = 5
'gameState:Int
'
'
'Const GAMESPEED:Float = 2.5
'lastTime:Float = 0
'timePassed:Float = 0
'delta:Float = 0
'deltaLow:Float = 0
'
'
'gridOn:Int
End
